@@ -3,7 +3,7 @@
  *
  * Copyright (c) 2001 Dug Song <dugsong@monkey.org>
  *
- * $Id: arp-ioctl.c,v 1.15 2002/01/20 21:23:27 dugsong Exp $
+ * $Id: arp-ioctl.c,v 1.20 2002/02/03 02:15:49 dugsong Exp $
  */
 
 #include "config.h"
@@ -31,7 +31,6 @@
 
 #include <stropts.h>
 #endif
-#include <assert.h>
 #include <errno.h>
 #include <fcntl.h>
 #include <stdio.h>
@@ -79,44 +78,50 @@ arp_open(void)
 
 #ifdef HAVE_ARPREQ_ARP_DEV
 static int
-arp_set_dev(const char *device, const struct intf_info *info, void *arg)
+_arp_set_dev(const struct intf_entry *entry, void *arg)
 {
 	struct arpreq *ar = (struct arpreq *)arg;
 	struct addr dst;
 	uint32_t mask;
+
+	if (entry->intf_type == INTF_TYPE_ETH && entry->intf_addr != NULL) {
+		addr_btom(entry->intf_addr->addr_bits, &mask, IP_ADDR_LEN);
+		addr_ston((struct sockaddr *)&ar->arp_pa, &dst);
 	
-	addr_btom(info->intf_addr.addr_bits, &mask, IP_ADDR_LEN);
-	addr_ston((struct sockaddr *)&ar->arp_pa, &dst);
-	
-	if ((info->intf_addr.addr_ip & mask) == (dst.addr_ip & mask)) {
-		strlcpy(ar->arp_dev, device, sizeof(ar->arp_dev));
-		return (1);
+		if ((entry->intf_addr->addr_ip & mask) ==
+		    (dst.addr_ip & mask)) {
+			strlcpy(ar->arp_dev, entry->intf_name,
+			    sizeof(ar->arp_dev));
+			return (1);
+		}
 	}
 	return (0);
 }
 #endif
 
 int
-arp_add(arp_t *a, const struct addr *pa, const struct addr *ha)
+arp_add(arp_t *a, const struct arp_entry *entry)
 {
 	struct arpreq ar;
 
 	memset(&ar, 0, sizeof(ar));
 
-	if (addr_ntos(pa, &ar.arp_pa) < 0 ||
-	    addr_ntos(ha, &ar.arp_ha) < 0)
+	if (addr_ntos(&entry->arp_pa, &ar.arp_pa) < 0)
 		return (-1);
 
 	/* XXX - see arp(7) for details... */
 #ifdef __linux__
+	if (addr_ntos(&entry->arp_ha, &ar.arp_ha) < 0)
+		return (-1);
 	ar.arp_ha.sa_family = ARP_HRD_ETH;
 #else
-	/* XXX - Solaris, HP-UX, others? */
+	/* XXX - Solaris, HP-UX, IRIX, other Mentat stacks? */
 	ar.arp_ha.sa_family = AF_UNSPEC;
+	memcpy(ar.arp_ha.sa_data, &entry->arp_ha.addr_eth, ETH_ADDR_LEN);
 #endif
 
 #ifdef HAVE_ARPREQ_ARP_DEV
-	if (intf_loop(a->intf, arp_set_dev, &ar) != 1) {
+	if (intf_loop(a->intf, _arp_set_dev, &ar) != 1) {
 		errno = ESRCH;
 		return (-1);
 	}
@@ -126,7 +131,7 @@ arp_add(arp_t *a, const struct addr *pa, const struct addr *ha)
 	/* XXX - screwy extended arpreq struct */
 	{
 		struct sockaddr_in *sin;
-		
+
 		ar.arp_hw_addr_len = ETH_ADDR_LEN;
 		sin = (struct sockaddr_in *)&ar.arp_pa_mask;
 		sin->sin_family = AF_INET;
@@ -142,7 +147,7 @@ arp_add(arp_t *a, const struct addr *pa, const struct addr *ha)
 		struct sockaddr_in sin;
 		int fd;
 		
-		addr_ntos(pa, (struct sockaddr *)&sin);
+		addr_ntos(&entry->arp_pa, (struct sockaddr *)&sin);
 		sin.sin_port = htons(666);
 		
 		if ((fd = socket(AF_INET, SOCK_DGRAM, 0)) < 0)
@@ -160,13 +165,13 @@ arp_add(arp_t *a, const struct addr *pa, const struct addr *ha)
 }
 
 int
-arp_delete(arp_t *a, const struct addr *pa)
+arp_delete(arp_t *a, const struct arp_entry *entry)
 {
 	struct arpreq ar;
 
 	memset(&ar, 0, sizeof(ar));
 	
-	if (addr_ntos(pa, &ar.arp_pa) < 0)
+	if (addr_ntos(&entry->arp_pa, &ar.arp_pa) < 0)
 		return (-1);
 	
 	if (ioctl(a->fd, SIOCDARP, &ar) < 0)
@@ -176,17 +181,17 @@ arp_delete(arp_t *a, const struct addr *pa)
 }
 
 int
-arp_get(arp_t *a, const struct addr *pa, struct addr *ha)
+arp_get(arp_t *a, struct arp_entry *entry)
 {
 	struct arpreq ar;
 
 	memset(&ar, 0, sizeof(ar));
 	
-	if (addr_ntos(pa, &ar.arp_pa) < 0)
+	if (addr_ntos(&entry->arp_pa, &ar.arp_pa) < 0)
 		return (-1);
 	
 #ifdef HAVE_ARPREQ_ARP_DEV
-	if (intf_loop(a->intf, arp_set_dev, &ar) != 1) {
+	if (intf_loop(a->intf, _arp_set_dev, &ar) != 1) {
 		errno = ESRCH;
 		return (-1);
 	}
@@ -198,7 +203,7 @@ arp_get(arp_t *a, const struct addr *pa, struct addr *ha)
 		errno = ESRCH;
 		return (-1);
 	}
-	return (addr_ston(&ar.arp_ha, ha));
+	return (addr_ston(&ar.arp_ha, &entry->arp_ha));
 }
 
 #ifdef HAVE_LINUX_PROCFS
@@ -206,7 +211,7 @@ int
 arp_loop(arp_t *a, arp_handler callback, void *arg)
 {
 	FILE *fp;
-	struct addr pa, ha;
+	struct arp_entry entry;
 	char buf[BUFSIZ], ipbuf[100], macbuf[100], maskbuf[100], devbuf[100];
 	int i, type, flags, ret;
 
@@ -221,9 +226,9 @@ arp_loop(arp_t *a, arp_handler callback, void *arg)
 		if (i < 4 || (flags & ATF_COM) == 0)
 			continue;
 		
-		if (addr_aton(ipbuf, &pa) == 0 &&
-		    addr_aton(macbuf, &ha) == 0) {
-			if ((ret = callback(&pa, &ha, arg)) != 0)
+		if (addr_aton(ipbuf, &entry.arp_pa) == 0 &&
+		    addr_aton(macbuf, &entry.arp_ha) == 0) {
+			if ((ret = callback(&entry, arg)) != 0)
 				break;
 		}
 	}
@@ -239,6 +244,7 @@ arp_loop(arp_t *a, arp_handler callback, void *arg)
 int
 arp_loop(arp_t *r, arp_handler callback, void *arg)
 {
+	struct arp_entry entry;
 	struct strbuf msg;
 	struct T_optmgmt_req *tor;
 	struct T_optmgmt_ack *toa;
@@ -298,8 +304,6 @@ arp_loop(arp_t *r, arp_handler callback, void *arg)
 		flags = 0;
 		
 		do {
-			struct addr pa, ha;
-			
 			rc = getmsg(r->fd, NULL, &msg, &flags);
 			
 			if (rc != 0 && rc != MOREDATA)
@@ -312,20 +316,21 @@ arp_loop(arp_t *r, arp_handler callback, void *arg)
 			arpend = (mib2_ipNetToMediaEntry_t *)
 			    (msg.buf + msg.len);
 
-			pa.addr_type = ADDR_TYPE_IP;
-			pa.addr_bits = IP_ADDR_BITS;
+			entry.arp_pa.addr_type = ADDR_TYPE_IP;
+			entry.arp_pa.addr_bits = IP_ADDR_BITS;
 			
-			ha.addr_type = ADDR_TYPE_ETH;
-			ha.addr_bits = ETH_ADDR_BITS;
+			entry.arp_ha.addr_type = ADDR_TYPE_ETH;
+			entry.arp_ha.addr_bits = ETH_ADDR_BITS;
 
 			for ( ; arp < arpend; arp++) {
-				pa.addr_ip = arp->ipNetToMediaNetAddress;
+				entry.arp_pa.addr_ip =
+				    arp->ipNetToMediaNetAddress;
 				
-				memcpy(&ha.addr_eth,
+				memcpy(&entry.arp_ha.addr_eth,
 				    arp->ipNetToMediaPhysAddress.o_bytes,
 				    ETH_ADDR_LEN);
 				
-				if ((ret = callback(&pa, &ha, arg)) != 0)
+				if ((ret = callback(&entry, arg)) != 0)
 					return (ret);
 			}
 		} while (rc == MOREDATA);
@@ -339,7 +344,7 @@ int
 arp_loop(arp_t *r, arp_handler callback, void *arg)
 {
 	struct nmparms nm;
-	struct addr ha, pa;
+	struct arp_entry entry;
 	mib_ipNetToMediaEnt arpentries[MAX_ARPENTRIES];
 	int fd, i, n, ret;
 	
@@ -357,25 +362,25 @@ arp_loop(arp_t *r, arp_handler callback, void *arg)
 	}
 	close_mib(fd);
 
-	pa.addr_type = ADDR_TYPE_IP;
-	pa.addr_bits = IP_ADDR_BITS;
+	entry.arp_pa.addr_type = ADDR_TYPE_IP;
+	entry.arp_pa.addr_bits = IP_ADDR_BITS;
 
-	ha.addr_type = ADDR_TYPE_ETH;
-	ha.addr_bits = ETH_ADDR_BITS;
+	entry.arp_ha.addr_type = ADDR_TYPE_ETH;
+	entry.arp_ha.addr_bits = ETH_ADDR_BITS;
 	
 	n /= sizeof(*arpentries);
 	ret = 0;
 	
 	for (i = 0; i < n; i++) {
-		if (arpentries[i].Type != INTM_DYNAMIC ||
+		if (arpentries[i].Type == INTM_INVALID ||
 		    arpentries[i].PhysAddr.o_length != ETH_ADDR_LEN)
 			continue;
 		
-		pa.addr_ip = arpentries[i].NetAddr;
-		memcpy(&ha.addr_eth, arpentries[i].PhysAddr.o_bytes,
+		entry.arp_pa.addr_ip = arpentries[i].NetAddr;
+		memcpy(&entry.arp_ha.addr_eth, arpentries[i].PhysAddr.o_bytes,
 		    ETH_ADDR_LEN);
 		
-		if ((ret = callback(&pa, &ha, arg)) != 0)
+		if ((ret = callback(&entry, arg)) != 0)
 			break;
 	}
 	return (ret);
@@ -392,8 +397,6 @@ arp_loop(arp_t *a, arp_handler callback, void *arg)
 arp_t *
 arp_close(arp_t *a)
 {
-	assert(a != NULL);
-	
 	if (a->fd > 0)
 		close(a->fd);
 #ifdef HAVE_ARPREQ_ARP_DEV
