@@ -3,7 +3,7 @@
  *
  * Copyright (c) 2001 Dug Song <dugsong@monkey.org>
  *
- * $Id: fw-pf.c,v 1.3 2001/12/29 21:06:41 dugsong Exp $
+ * $Id: fw-pf.c,v 1.8 2002/01/20 21:23:28 dugsong Exp $
  */
 
 #include "config.h"
@@ -31,40 +31,43 @@ struct fw_handle {
 };
 
 static void
-fr_to_pr(struct fw_rule *fr, struct pf_rule *pr)
+fr_to_pr(const struct fw_rule *fr, struct pf_rule *pr)
 {
 	memset(pr, 0, sizeof(*pr));
 	
-	strlcpy(pr->ifname, fr->device, sizeof(pr->ifname));
+	strlcpy(pr->ifname, fr->fw_device, sizeof(pr->ifname));
 	
-	pr->action = (fr->op == FW_OP_ALLOW) ? PF_PASS : PF_DROP;
-	pr->direction = (fr->direction == FW_DIR_IN) ? PF_IN : PF_OUT;
-	pr->proto = fr->proto;
+	pr->action = (fr->fw_op == FW_OP_ALLOW) ? PF_PASS : PF_DROP;
+	pr->direction = (fr->fw_dir == FW_DIR_IN) ? PF_IN : PF_OUT;
+	pr->proto = fr->fw_proto;
 
-	pr->src.addr.v4.s_addr = fr->src.addr_ip;
-	addr_btom(fr->src.addr_bits, &pr->src.mask.v4.s_addr, IP_ADDR_LEN);
+	pr->af = AF_INET;
+	pr->src.addr.v4.s_addr = fr->fw_src.addr_ip;
+	addr_btom(fr->fw_src.addr_bits, &pr->src.mask.v4.s_addr, IP_ADDR_LEN);
 	
-	pr->dst.addr.v4.s_addr = fr->dst.addr_ip;
-	addr_btom(fr->dst.addr_bits, &pr->dst.mask.v4.s_addr, IP_ADDR_LEN);
+	pr->dst.addr.v4.s_addr = fr->fw_dst.addr_ip;
+	addr_btom(fr->fw_dst.addr_bits, &pr->dst.mask.v4.s_addr, IP_ADDR_LEN);
 	
-	switch (fr->proto) {
+	switch (fr->fw_proto) {
 	case IP_PROTO_ICMP:
-		if (fr->sport[1])
-			pr->type = (u_char)(fr->sport[0] & fr->sport[1]) + 1;
-		if (fr->dport[1])
-			pr->code = (u_char)(fr->dport[0] & fr->dport[1]) + 1;
+		if (fr->fw_sport[1])
+			pr->type = (u_char)(fr->fw_sport[0] &
+			    fr->fw_sport[1]) + 1;
+		if (fr->fw_dport[1])
+			pr->code = (u_char)(fr->fw_dport[0] &
+			    fr->fw_dport[1]) + 1;
 		break;
 	case IP_PROTO_TCP:
 	case IP_PROTO_UDP:
-		pr->src.port[0] = htons(fr->sport[0]);
-		pr->src.port[1] = htons(fr->sport[1]);
+		pr->src.port[0] = htons(fr->fw_sport[0]);
+		pr->src.port[1] = htons(fr->fw_sport[1]);
 		if (pr->src.port[0] == pr->src.port[1])
 			pr->src.port_op = PF_OP_EQ;
 		else
 			pr->src.port_op = PF_OP_IRG;
 
-		pr->dst.port[0] = htons(fr->dport[0]);
-		pr->dst.port[1] = htons(fr->dport[1]);
+		pr->dst.port[0] = htons(fr->fw_dport[0]);
+		pr->dst.port[1] = htons(fr->fw_dport[1]);
 		if (pr->dst.port[0] == pr->dst.port[1])
 			pr->dst.port_op = PF_OP_EQ;
 		else
@@ -74,52 +77,55 @@ fr_to_pr(struct fw_rule *fr, struct pf_rule *pr)
 }
 
 static int
-pr_to_fr(struct pf_rule *pr, struct fw_rule *fr)
+pr_to_fr(const struct pf_rule *pr, struct fw_rule *fr)
 {
 	memset(fr, 0, sizeof(*fr));
 	
-	strlcpy(fr->device, pr->ifname, sizeof(fr->device));
+	strlcpy(fr->fw_device, pr->ifname, sizeof(fr->fw_device));
 
 	if (pr->action == PF_DROP)
-		fr->op = FW_OP_BLOCK;
+		fr->fw_op = FW_OP_BLOCK;
 	else if (pr->action == PF_PASS)
-		fr->op = FW_OP_ALLOW;
+		fr->fw_op = FW_OP_ALLOW;
 	else
 		return (-1);
 	
-	fr->direction = pr->direction == PF_IN ? FW_DIR_IN : FW_DIR_OUT;
-	fr->proto = pr->proto;
+	fr->fw_dir = pr->direction == PF_IN ? FW_DIR_IN : FW_DIR_OUT;
+	fr->fw_proto = pr->proto;
 
-	fr->src.addr_type = ADDR_TYPE_IP;
-	addr_mtob(&pr->src.mask.v4.s_addr, IP_ADDR_LEN, &fr->src.addr_bits);
-	fr->src.addr_ip = pr->src.addr.v4.s_addr;
+	if (pr->af != AF_INET)
+		return (-1);
 	
- 	fr->dst.addr_type = ADDR_TYPE_IP;
-	addr_mtob(&pr->dst.mask.v4.s_addr, IP_ADDR_LEN, &fr->dst.addr_bits);
-	fr->dst.addr_ip = pr->dst.addr.v4.s_addr;
+	fr->fw_src.addr_type = ADDR_TYPE_IP;
+	addr_mtob(&pr->src.mask.v4.s_addr, IP_ADDR_LEN, &fr->fw_src.addr_bits);
+	fr->fw_src.addr_ip = pr->src.addr.v4.s_addr;
 	
-	switch (fr->proto) {
+ 	fr->fw_dst.addr_type = ADDR_TYPE_IP;
+	addr_mtob(&pr->dst.mask.v4.s_addr, IP_ADDR_LEN, &fr->fw_dst.addr_bits);
+	fr->fw_dst.addr_ip = pr->dst.addr.v4.s_addr;
+	
+	switch (fr->fw_proto) {
 	case IP_PROTO_ICMP:
 		if (pr->type) {
-			fr->sport[0] = pr->type - 1;
-			fr->sport[1] = 0xff;
+			fr->fw_sport[0] = pr->type - 1;
+			fr->fw_sport[1] = 0xff;
 		}
 		if (pr->code) {
-			fr->dport[0] = pr->code - 1;
-			fr->dport[1] = 0xff;
+			fr->fw_dport[0] = pr->code - 1;
+			fr->fw_dport[1] = 0xff;
 		}
 		break;
 	case IP_PROTO_TCP:
 	case IP_PROTO_UDP:
-		fr->sport[0] = ntohs(pr->src.port[0]);
-		fr->sport[1] = ntohs(pr->src.port[1]);
+		fr->fw_sport[0] = ntohs(pr->src.port[0]);
+		fr->fw_sport[1] = ntohs(pr->src.port[1]);
 		if (pr->src.port_op == PF_OP_EQ)
-			fr->sport[1] = fr->sport[0];
+			fr->fw_sport[1] = fr->fw_sport[0];
 
-		fr->dport[0] = ntohs(pr->dst.port[0]);
-		fr->dport[1] = ntohs(pr->dst.port[1]);
+		fr->fw_dport[0] = ntohs(pr->dst.port[0]);
+		fr->fw_dport[1] = ntohs(pr->dst.port[1]);
 		if (pr->dst.port_op == PF_OP_EQ)
-			fr->dport[1] = fr->dport[0];
+			fr->fw_dport[1] = fr->fw_dport[0];
 	}
 	return (0);
 }
@@ -132,15 +138,14 @@ fw_open(void)
 	if ((fw = calloc(1, sizeof(*fw))) == NULL)
 		return (NULL);
 
-	if ((fw->fd = open("/dev/pf", O_RDWR)) < 0) {
-		free(fw);
-		return (NULL);
-	}
+	if ((fw->fd = open("/dev/pf", O_RDWR)) < 0)
+		return (fw_close(fw));
+	
 	return (fw);
 }
 
 int
-fw_add(fw_t *fw, struct fw_rule *rule)
+fw_add(fw_t *fw, const struct fw_rule *rule)
 {
 	struct pfioc_changerule pcr;
 	
@@ -154,7 +159,7 @@ fw_add(fw_t *fw, struct fw_rule *rule)
 }
 
 int
-fw_delete(fw_t *fw, struct fw_rule *rule)
+fw_delete(fw_t *fw, const struct fw_rule *rule)
 {
 	struct pfioc_changerule pcr;
 	
@@ -172,7 +177,7 @@ fw_loop(fw_t *fw, fw_handler callback, void *arg)
 {
 	struct pfioc_rule pr;
 	struct fw_rule fr;
-	u_int32_t n, max;
+	uint32_t n, max;
 	int ret;
 	
 	if (ioctl(fw->fd, DIOCGETRULES, &pr) < 0)
@@ -193,14 +198,13 @@ fw_loop(fw_t *fw, fw_handler callback, void *arg)
 	return (0);
 }
 
-int
+fw_t *
 fw_close(fw_t *fw)
 {
 	assert(fw != NULL);
 
-	if (close(fw->fd) < 0)
-		return (-1);
-	
+	if (fw->fd > 0)
+		close(fw->fd);
 	free(fw);
-	return (0);
+	return (NULL);
 }
